#!/bin/sh

# web/bin/fcgirunner.pl

# Ensure ZEUSHOME is setup to point to our installation
if [ -z "$ZEUSHOME" ]
then
   # Have a good guess, we live in $ZEUSHOME/webadmin/bin
   OLD_PWD=`pwd`
   cd `dirname $0`/../..
   ZEUSHOME=`pwd`
   export ZEUSHOME
   cd $OLD_PWD
fi

if [ ! -d "$ZEUSHOME/web" ]
then
   echo 'FATAL: $ZEUSHOME is not set, or is set incorrectly'
   echo 'It should point to the Zeus installation running the adminserver'
   exit 1
fi

exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}

#----------------------------------------------------------
#!perl -w

# Copyright (C) 1995 - 2001 Zeus Technology Limited
#
# This program is provided as part of the Zeus Server product available from
# Zeus Technology Limited.  It is intended to provide an example of how
# to startup remote FastCGI processes which can interact with the Zeus
# Web Server FastCGI module.  This program, and any derived works may be
# used freely by customers for the purpose to which it is intended.
#
# This software is provided "as is" and any express or implied warranties, 
# including but not limited to, the implied warranties of merchantability and
# fitness for a particular purpose are disclaimed.  In no event shall 
# Zeus Technology Limited. be liable for any direct, indirect, incidental,
# special, exemplary, or consequential damaged (including, but not
# limited to, procurement of substitute good or services; loss of use,
# data, or profits; or business interruption) however caused and on
# theory of liability.  Whether in contract, strict liability or tort
# (including negligence or otherwise) arising in any way out of the use
# of this software, even if advised of the possibility of such damage
#
# For further information regarding this piece of software, please email
# support@zeus.com

BEGIN { unshift @INC, "$ENV{ZEUSHOME}/perl"; }
require 5;
use ZeusOS;

my $fcgi;
my $port;
my $addr  = "0.0.0.0";
my $user  = $>;
my $group = $);
my $pidfile;

# parse arguments
while( my $arg = shift @ARGV ) {
   $arg =~ /^--addr=/    and do { $addr    = $'; next; };
   $arg =~ /^--user=/    and do { $user    = $'; next; };
   $arg =~ /^--group=/   and do { $group   = $'; next; };
   $arg =~ /^--pidfile=/ and do { $pidfile = $'; next; };
   $arg =~ /^--help/     and do { help(); exit 0; };
   $port = $arg; $fcgi = $ARGV[0]; last;
}

if( ! defined $fcgi || ! defined $port ) {
   usage(); exit 1;
}
if( $port !~ /^\d+$/ ) {
   die "fcgirunner: port '$port' is not a valid port number\n";
}


# Set our permissions.
#
my $uid = ( "$user"  =~ /^\d+$/ )?$user :getpwnam( $user  )
   or die "fcgirunner: user '$user' is not a valid user name or user id\n";
my $gid = ( "$group" =~ /^(\d+)/ )?$1:getgrnam( $group )
   or die "fcgirunner: group '$group' is not a valid group name or group id\n";

( $(, $) ) = ( $gid, "$gid $gid" );
( $<, $> ) = ( $uid, $uid );

( $( == $gid and $) =~ /^$gid / and $< == $uid and $> == $uid )
   or die "fcgirunner: could not change privileges to $user, $group; ".
          "you should be root\n".
          "   UID = $<; EUID = $>; GID = $(; EGID = $(.\n";


# Check that the application exists, and that we can execute it.
#
( -x $fcgi ) or die "fcgirunner: FastCGI application '$fcgi' does not exist\n";


# Bind our server socket, and dup it onto STDIN.
#
my $bindaddr = ( $addr =~ /^\d+\.\d+\.\d+\.\d+$/ )?
                 pack( "C4", split /\./, $addr):
                 (gethostbyname $addr)[4]
   or die "fcgirunner: addr '$addr' is not a valid ip or host address\n";

socket( SERVER, $ZeusOS::PF_INET, $ZeusOS::SOCK_STREAM, getprotobyname('tcp') )
   or die "socket: $!\n";
setsockopt( SERVER, $ZeusOS::SOL_SOCKET, $ZeusOS::SO_REUSEADDR, 1 )
   or die "setsockopt( SO_REUSEADDR ): $!\n";

bind( SERVER, ZeusOS::pack_sockaddr_in( $port, $bindaddr ) )
   or die "bind( $addr:$port ): $!\n";
listen( SERVER, 128 ) or die "listen: $!\n";

open( STDIN, "<&=SERVER" ) or die "dup2 failed: $!\n";

close SERVER;


# Create the fcgi child process and exit.
#
my $pid = fork;
( $pid >= 0 ) or die "fork: $!\n";

if( $pid == 0 ) {
   # Child.  Set process group to $pid, so signals sent to '-$pid' will go
   # to this child and all its children
   setpgrp( 0, 0 ) or die "setpgrp( 0, 0 ): $!\n";
   
   exec( @ARGV ) or die "exec( ".(join ", ", @ARGV )." ) failed: $!\n";
}

# Parent.  Write pidfile if necessary, then exit.
if( defined $pidfile ) {
   if( $pidfile eq "-" ) {
      print "$pid\n"
   } else {
      open OUT, ">$pidfile";
      print OUT "$pid\n";
      close OUT;
   }
}

exit( 0 );


#-----------------------------------------------------------------------------#

sub usage()
{
   print STDERR <<EOM;
Usage: 
        fcgirunner [--addr=<bind_address>] [--user=<uid>] [--group=<gid>]
                   [--pidfile=<file_to_save_pid>]
                   <port> <fastcgi_program> [<args_for_fastcgi_program>]
More Information:
        fcgirunner --help
EOM
}

sub help()
{
   usage();
   print STDERR <<EOM;

--addr=<bind_address>
        Specify the ip address to bind the FastCGI application to.
        Defaults to all local interfaces (BIND_ANY).
--user=<uid>
--group=<gid>
        Specify the user and group to run the FastCGI application as.
        Defaults to the current user and group.
--pidfile=<file_to_save_pid>
        Specify the file to save the pid (process ID) of the FastCGI
        application process.  Specify '-' to print the pid to stdout.
        The fcgirunner sets its process group to the pid, so the FastCGI
        application can normally be terminated by "kill -TERM -<pid>".
<port>
        Specify the port to bind the FastCGI application to.
<fastcgi_program>
        Specify the filename of the FastCGI application.
<args_for_fastcgi_program>
        Specify any additional command line arguments to pass to the
        FastCGI program.
EOM
}
